package xyz.tbvns.Models;

import lombok.extern.slf4j.Slf4j;
import org.joml.Vector4f;

import java.awt.*;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.concurrent.atomic.AtomicReference;

@Slf4j
public class ReadOBJ {
    public ArrayList<Vector4f> getPoints(File OBJFile) throws FileNotFoundException {
        BufferedReader reader = new BufferedReader(new FileReader(OBJFile));
        ArrayList<Vector4f> Points = new ArrayList<>();
        reader.lines().forEach(l -> {
            if (l.startsWith("v ")) {
                String substring = l.substring(2);
                String[] cord = substring.split(" ");
                Vector4f vector4f = new Vector4f();
                vector4f.x = Float.parseFloat(cord[0]);
                vector4f.y = Float.parseFloat(cord[1]);
                vector4f.z = Float.parseFloat(cord[2]);
                Points.add(vector4f);
            }
        });
        return Points;
    }

    public ArrayList<Face> getFace(File OBJFile, File MTLFile) throws FileNotFoundException {
        BufferedReader reader = new BufferedReader(new FileReader(OBJFile));
        ArrayList<Face> Faces = new ArrayList<>();
        AtomicReference<HashMap<String, Color>> materials = new AtomicReference<>(new HashMap<>());
        AtomicReference<Color> color = new AtomicReference<>(new Color(0, 0, 0));
        reader.lines().forEach(l -> {
                try {
                    materials.set(ReadMTL.getColors(MTLFile));
                } catch (FileNotFoundException e) {
                    throw new RuntimeException(e);
                }

                if (l.startsWith("usemtl")) {
                    color.set(materials.get().get(l.split(" ")[1]));
                }

            if (l.startsWith("f")) {
                Face face = new Face();
                String substring = l.substring(2);
                String[] cord = substring.split(" ");
                if (cord.length == 3) {
                    for (int i = 0; i < cord.length; i++) {
                        String id = cord[i].split("/")[0];
                        face.points.add(Integer.valueOf(id));
                    }

                    face.color = color.get();

                    Faces.add(face);
                } else if (cord.length == 4) {
                    for (int i = 0; i < cord.length; i++) {
                        String id = cord[i].split("/")[0];
                        face.points.add(Integer.valueOf(id));
                    }
                    face.Has4Point = true;

                    face.color = color.get();

                    Faces.add(face);
                } else {
                    log.error("A face had more than 4 point, ignoring it");
                }
            }
        });
        return Faces;
    }

    public ArrayList<Face> getFaceAnimation(File OBJFile, File MTLFile) throws FileNotFoundException {
        BufferedReader reader = new BufferedReader(new FileReader(OBJFile));
        ArrayList<Face> Faces = new ArrayList<>();
        AtomicReference<HashMap<String, Color>> materials = new AtomicReference<>(new HashMap<>());
        AtomicReference<Color> color = new AtomicReference<>(new Color(0, 0, 0));
        reader.lines().forEach(l -> {
            try {
                materials.set(ReadMTL.getColors(MTLFile));
            } catch (FileNotFoundException e) {
                throw new RuntimeException(e);
            }
            if (l.startsWith("usemtl")) {
                color.set(materials.get().get(l.split(" ")[1]));
            }
            if (l.startsWith("f")) {
                Face face = new Face();
                String substring = l.substring(2);
                String[] cord = substring.split(" ");
                if (cord.length == 3) {
                    for (int i = 0; i < cord.length; i++) {
                        String id = cord[i].split("/")[0];
                        face.points.add(Integer.valueOf(id));
                    }

                    face.color = color.get();

                    Faces.add(face);
                } else if (cord.length == 4) {
                    for (int i = 0; i < cord.length; i++) {
                        String id = cord[i].split("/")[0];
                        face.points.add(Integer.valueOf(id));
                    }
                    face.Has4Point = true;

                    face.color = color.get();

                    Faces.add(face);
                } else {
                    log.error("A face had more than 4 point, ignoring it");
                }
            }
        });
        return Faces;
    }
}
