package xyz.tbvns.ui;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.InputProcessor;
import com.badlogic.gdx.backends.lwjgl3.Lwjgl3Graphics;
import imgui.ImGui;
import imgui.ImGuiIO;
import imgui.flag.ImGuiWindowFlags;
import imgui.gl3.ImGuiImplGl3;
import imgui.glfw.ImGuiImplGlfw;
import lombok.Setter;
import xyz.tbvns.Main;

public class Ui {
    @Setter
    private static boolean showLevelSelector = true;

    static private ImGuiImplGlfw imGuiGlfw;
    static private ImGuiImplGl3 imGuiGl3;
    private static InputProcessor tmpProcessor;

    public static void render() {
        startImGui();

        //TODO: Add functionality
        ImGui.beginMainMenuBar();

        if (ImGui.beginMenu("Files")) {
            ImGui.menuItem("Save", "CTRL S");
            ImGui.menuItem("Export", "SHIFT S");
            if (ImGui.menuItem("Open", "O")) {
                //TODO: Add save dialog if the level was modified
                showLevelSelector = true;
                Main.modelCache = null;
            }
            ImGui.menuItem("Close", "C");
            ImGui.endMenu();
        }

        if (ImGui.beginMenu("View")) {
            ImGui.checkbox("Level grid", true);
            ImGui.checkbox("World grid", true);
            ImGui.endMenu();
        }

        if (ImGui.beginMenu("Tools")) {
            ImGui.menuItem("Edit level string", "L");
            ImGui.menuItem("BPM to FPS", "B");
            ImGui.endMenu();
        }

        if (ImGui.beginMenu("About")) {
            ImGui.menuItem("Help", "H");
            ImGui.menuItem("Wiki", "CTRL - H");
            ImGui.menuItem("Info", "I");
            ImGui.menuItem("Discord");
            ImGui.menuItem("Git");
            ImGui.endMenu();
        }


        ImGui.endMainMenuBar();

        ImGui.begin("Menu", ImGuiWindowFlags.NoDecoration | ImGuiWindowFlags.NoMove);
        ImGui.setWindowSize(200, Gdx.graphics.getHeight() - 19 - 200);
        ImGui.setWindowPos(0, 19);
        ImGui.end();

        ImGui.begin("TimeMenu", ImGuiWindowFlags.NoDecoration | ImGuiWindowFlags.NoMove);
        ImGui.setWindowSize(Gdx.graphics.getWidth(), 201);
        ImGui.setWindowPos(0, Gdx.graphics.getHeight() - 201);
        ImGui.end();

        if (showLevelSelector) {
            LevelSelector.render();
        }

        endImGui();
    }

    public static void initImGui() {
        imGuiGlfw = new ImGuiImplGlfw();
        imGuiGl3 = new ImGuiImplGl3();
        long windowHandle = ((Lwjgl3Graphics) Gdx.graphics).getWindow().getWindowHandle();
        ImGui.createContext();
        ImGuiIO io = ImGui.getIO();
        io.setIniFilename(null);
        io.getFonts().addFontDefault();
        io.getFonts().build();
        imGuiGlfw.init(windowHandle, true);
        imGuiGl3.init("#version 150");
    }

    public static void startImGui() {
        if (tmpProcessor != null) { // Restore the input processor after ImGui caught all inputs, see #end()
            Gdx.input.setInputProcessor(tmpProcessor);
            tmpProcessor = null;
        }

        imGuiGlfw.newFrame();
        ImGui.newFrame();
    }

    public static void endImGui() {
        ImGui.render();
        imGuiGl3.renderDrawData(ImGui.getDrawData());

        // If ImGui wants to capture the input, disable libGDX's input processor
        if (ImGui.getIO().getWantCaptureKeyboard() || ImGui.getIO().getWantCaptureMouse()) {
            tmpProcessor = Gdx.input.getInputProcessor();
            Gdx.input.setInputProcessor(null);
        }
    }

    public static void dispose() {
        imGuiGl3.dispose();
        imGuiGl3 = null;
        imGuiGlfw.dispose();
        imGuiGlfw = null;
        ImGui.destroyContext();
    }
}
